#include "Display.h"
#include "DecodeEthernet.h"
#include "Time.h"

Display::Display(Messages * messages){
	this->m_pMessages = messages;
}

void Display::DisplayALL(){
	for (int i =0 ; i < m_pMessages->Count(); i++){
		DisplayMessage(m_pMessages->operator[](i));
	}
}

void Display::DisplayMessage(int index){
	DisplayMessage(m_pMessages->operator[](index));
}

void Display::DisplayMessage(icsSpyMessage * message){
	
	switch(message->Protocol){
		case SPY_PROTOCOL_CAN:
			DisplayCAN(message);
			break;
		case SPY_PROTOCOL_ETHERNET:
			DisplayEthernet(message);
			break;
		case SPY_PROTOCOL_CANFD:
			DisplayCANFD(message);
			break;
		//TODO
	}

}

void Display::DisplayCAN(icsSpyMessage * message){
	
	printf("Protocal: CAN\tNetwork: %s\t",   getNetworkString(message->NetworkID ));
	PrintTime(message);
	printf("\n\tARBID: %d Length: %d\t Bytes:", message->ArbIDOrHeader, message->NumberBytesData);

	PrintHex(message->Data,  message->NumberBytesData, true); // can does not use extended data pointer to keep data
	printf("\n\n");
}

void Display::DisplayCANFD(icsSpyMessage * message){
	printf("Protocal: CANFD\tNetwork: %s\t",getNetworkString(message->NetworkID));
	PrintTime(message);
	printf("\n\tARBID: %d\tLength: %d\t Bytes:",
		 message->ArbIDOrHeader, message->NumberBytesData); 
	

	pExtraDataPtrHandler * EDPData = (pExtraDataPtrHandler *)(message->ExtraDataPtr);
	PrintHex((unsigned char *)EDPData->pExtraDataPtr, EDPData->length, true);// add data for can fd is in extended data pointer
	printf("\n\n");
}

void Display::DisplayEthernet(icsSpyMessage * message){
	
	int protocol = IdentifyEthernet(message); //the extended data pointer contains raw frame of message
	switch (protocol){
	case eEthernet:
		PrintEthernet(message);
		break;
	case eIPv4:
	case eIPV4ProtocolHOPOPT:
	case eIPV4ProtocolITCP:
	case eIPV4ProtocolUDP:
	case eIPV4ProtocolICMP:
	case eIPV4ProtocolIGMP:
		{
		PrintEthernet(message);
		PrintIPv4(message);
		break;
		}
	case eIPv6:
	case eIPV4ProtocolICMPv6:{
		PrintEthernet(message);
		PrintIPv6(message);
		break;
		}
	//TODO
	default://need to add display for other protocals
		PrintEthernet(message);
		if(protocol > 1)
			printf("\tdiplay not present\n");
		else
			printf("\tinvalid Type\n");
	}

	
	printf("\n\n");
}


unsigned int Display::IdentifyEthernet(icsSpyMessage * message){ // used this to determine what to print

	if (message->Protocol != SPY_PROTOCOL_ETHERNET)
		return 0;
	pExtraDataPtrHandler * EDPData = (pExtraDataPtrHandler *)(message->ExtraDataPtr);
	EthernetHeader * ethernetHeader = (EthernetHeader *)EDPData->pExtraDataPtr;
	unsigned short  EtherType =  (ethernetHeader->EtherType[0] << 8) | ethernetHeader->EtherType[1]; 
	
	if (EtherType == (0x0000))
		return eEthernet;
	else if (EtherType == (0x0800)){
		IPv4Header * ipv4Header = (IPv4Header *)EDPData->pExtraDataPtr;
		if ((ipv4Header->Protocol) == (0x00))
			return eIPV4ProtocolHOPOPT;
		else if ((ipv4Header->Protocol) == (0x06))
			return eIPV4ProtocolITCP;
		else if ((ipv4Header->Protocol) == (0x11))
			return eIPV4ProtocolUDP;
		else if ((ipv4Header->Protocol) == (0x01))
			return eIPV4ProtocolICMP;
		else if ((ipv4Header->Protocol) == (0x58))
			return eIPV4ProtocolICMPv6;
		else if ((ipv4Header->Protocol) == (0x02))
			return eIPV4ProtocolIGMP;
		else if ((ipv4Header->Protocol) == (0xFF))
			return eIPv4;
	}
	else if (EtherType == (0x0806))
		return eARP;
	else if (EtherType == (0x86DD))
		return eIPv6;	
	else if (EtherType== (0x22f0))
		return eIEEE1722;
	else if (EtherType == (0x88F7))
		return ePTP;
	else if (EtherType == (0x8100))
		return eVlANInner;
	else if (EtherType == (0x88A8))
		return eVlanOuter;
	return 0;
}

void Display::PrintEthernet(icsSpyMessage * message){
	printf("Protocal: Ethernet\tNetwork: %s\t", getNetworkString(message->NetworkID));
	PrintTime(message);
	printf("\n");

	EthernetHeader * etherHeader = (EthernetHeader *)((pExtraDataPtrHandler *)(message->ExtraDataPtr))->pExtraDataPtr;
	printf("  Destination: ");
	PrintMacAddress(etherHeader->Destination, 6);
	printf("\tSource: ");
	PrintMacAddress(etherHeader->Source, 6);
	printf("\tType: ");
	PrintHex(etherHeader->EtherType, 2, false);
	printf("\n");
}

void Display::PrintIPv4(icsSpyMessage * message){
	IPv4Header * ipv4Header = (IPv4Header *)((pExtraDataPtrHandler *)(message->ExtraDataPtr))->pExtraDataPtr;
	printf("  IP Destination: ");
	PrintIPAddress(ipv4Header->DestinationAddress, 4);
	printf("\tIP Source: ");
	PrintIPAddress((unsigned char * )&ipv4Header->SourceAddress, 4);
	printf("\tType: ");
	PrintHex(&ipv4Header->Protocol, 1, false);
	printf("\n");
}

void Display::PrintIPv6(icsSpyMessage * message){
	IPv6Header * ipPv6Header = (IPv6Header *)((pExtraDataPtrHandler *)(message->ExtraDataPtr))->pExtraDataPtr;
	printf("  IP Destination: ");
	PrintIPAddress(ipPv6Header->DestinationAddress, 16);
	printf("\tIP Source: ");
	PrintIPAddress(ipPv6Header->SourceAddress, 16);
	printf("\tType: ");
	PrintHex(&ipPv6Header->NextHeader, 1, false);
	printf("\n");
}

void Display::PrintHex(unsigned char * value, int size, bool indent){
	if (size < 1)
		return;
	printf("%02X%s" , value[0], (indent)?" ":"");
	for (int i = 1; i < size; i++)
		printf("%02X%s" , value[i], (indent)?" ":"");	
}


void Display::PrintMacAddress(unsigned char * value, int size){
	if (size < 1)
		return;
	printf("%02X", value[0]);
	for (int i = 1; i < size; i++)
		printf(".%02X", value[i]);
}

void Display::PrintIPAddress(unsigned char * value, int size){
	if (size < 1)
		return;
	printf("%02d", value[0]);
	for (int i = 1; i < size; i++)
		printf(".%02d", value[i]);
}

void Display::PrintTime(icsSpyMessage * message ){
	SDateTime time = GetSpyMessageTimeSec(*message);
	printf("%d/%d/%d   %2d:%02d:%02d:%03d%03d", time.Year, time.Month, time.Day, time.Hour, time.Min, time.Sec, time.msec, time.usec);
}
char * Display::getNetworkString(unsigned char networkID){
	switch (networkID){
	case NETID_HSCAN:
		return "HS_CAN";
	case NETID_MSCAN:
		return "MS_CAN";
	case NETID_SWCAN:
		return "SW_CAN";
	case NETID_LSFTCAN:
		return "LSFT_CAN";	
	case NETID_FORDSCP:
		return "FORDSCP";	
	case NETID_J1708:
		return "J1708";	
	case NETID_JVPW:
		return "JVPW";	
	case NETID_ISO:
		return "ISO";	
	case NETID_ISO2:
		return "ISO2";	
	case NETID_ISO14230:
		return "ISO14230";	
	case NETID_LIN:
		return "LIN";	
	case NETID_ISO3:
		return "ISO3";	
	case NETID_HSCAN2:
		return "HSCAN2";	
	case NETID_HSCAN3:
		return "HSCAN3";
	case NETID_ISO4:
		return "ISO4";	
	case NETID_LIN2:
		return "LIN2";	
	case NETID_LIN3:
		return "LIN3";	
	case NETID_LIN4:
		return "LIN4";	
	case NETID_LIN5:
		return "LIN5";	
	case NETID_MOST:
		return "MOST";	
	case NETID_CGI:
		return "CGI";	
	case NETID_HSCAN4:
		return "HSCAN4";	
	case NETID_HSCAN5:
		return "HSCAN5";
	case NETID_UART:
		return "UART";
	case NETID_UART2:
		return "UART2";
	case NETID_UART3:
		return "UART3";
	case NETID_UART4:
		return "UART4";
	case NETID_SWCAN2: //changed from ANALOG;
		return "SWCAN2";
	case NETID_ETHERNET_DAQ: //NETID_WIZNET - plasma/ ion daq port ethernet;
		return "ETHERNET_DAQ";
	case NETID_TEXTAPI_TO_HOST:
		return "TEXTAPI_TO_HOST";
	case NETID_FLEXRAY1A:
		return "FLEXRAY1A";
	case NETID_FLEXRAY1B:
		return "FLEXRAY1B";
	case NETID_FLEXRAY2A:
		return "FLEXRAY2A";
	case NETID_FLEXRAY2B:
		return "FLEXRAY2B";
	case NETID_FLEXRAY:
		return "FLEXRAY";
	case NETID_MOST25:
		return "MOST25";
	case NETID_MOST50:
		return "MOST50";
	case NETID_MOST150:
		return "MOST150";
	case NETID_ETHERNET:
		return "ETHERNET";
	case NETID_GMFSA:
		return "GMFSA";
	case NETID_TCP:
		return "TCP";
	case NETID_HSCAN6:
		return "HSCAN6";
	case NETID_HSCAN7:
		return "HSCAN7";
	case NETID_LIN6:
		return "LIN6";
	case NETID_LSFTCAN2:
		return "LSFTCAN2";
	case NETID_OP_ETHERNET1:
		return "OP_ETHERNET1";
	case NETID_OP_ETHERNET2:
		return "OP_ETHERNET2";
	case NETID_OP_ETHERNET3:
		return "OP_ETHERNET3";
	case NETID_OP_ETHERNET4:
		return "OP_ETHERNET4";
	case NETID_OP_ETHERNET5:
		return "OP_ETHERNET5";
	case NETID_OP_ETHERNET6:
		return "OP_ETHERNET6";
	case NETID_OP_ETHERNET7:
		return "OP_ETHERNET7";
	case NETID_OP_ETHERNET8:
		return "OP_ETHERNET8";
	case NETID_OP_ETHERNET9:
		return "OP_ETHERNET9";
	case NETID_OP_ETHERNET10:
		return "OP_ETHERNET10";
	case NETID_OP_ETHERNET11:
		return "OP_ETHERNET11";
	case NETID_OP_ETHERNET12:
		return "OP_ETHERNET12";
	default:
		return "INVALID";
	}
}


SDateTime Display::GetSpyMessageTimeSec(const icsSpyMessage & stMsg ) { // returns time struct
	unsigned long long ttt = stMsg.TimeHardware2;
    ttt <<= 32;
    ttt |= stMsg.TimeHardware;
	SDateTime RTC = Time::GetRTC(ttt);
	return RTC;
}


