#include "Time.h"


Time::Time(void)
{
}


Time::~Time(void)
{
}

SDateTime Time::GetRTC (long long int T){
	SDateTime rtcTime = {0};
	RTC_TicksToRTC (T, &rtcTime );
	return rtcTime;
}

long long int Time::RTC_TicksToRTC (long long int T, SDateTime	*rtc )
{

	long long int  ND, NH, NM, NS, NMN, NY, ms, us;
	long long int  four_yr=0,yr1=0;

	four_yr = D64( T, FourYearsN );
	T=T-M64( FourYearsN, four_yr );		// --- The rest of ticks

  	/* is there enough time for a first year? */
	if(T >= Year365N)
	{  	/* This next candidate year is 2007 + 4 X n, for some n. */
		yr1++; //move cursor to 2001 so sum 2000(leap year)
		T -= Year365N;

        /* is there enough time for a second year? */
        if(T >= Year366N) /* leap yr */
        {	/* This next candidate year is 2008 + 4 X n, for some n. */
            yr1++;
            T -= Year366N; /* leap yr */
            
        	/* is there enough time for a third year? */
            if(T >= Year365N)
            { /* This next candidate year is 2009 + 4 X n, for some n. */
                yr1++;
                T -= Year365N;
                                   
	        	/* is there enough time for a forth year? */
                if(T >= Year365N)
                {
                    yr1++; // sum 2003
                    T -= Year365N;//ticks in year 2000
                }
            }
        }
	}
		
	NY = four_yr*4 + yr1;   
	rtc->Year = ( NY ) + 2007;
	
	T = RTC_TicksToMonths(T,&NMN,rtc->Year);

	ND=D64( T, DayN );			// --- Day, Day Zero is first day of the month
	T=T-M64( DayN, ND );		// --- The rest of ticks    
    ++ND; /* Day Zero => 1st */

	NH=D64( T, HourN );			// --- Hours
	T=T-M64( HourN, NH );		// --- The rest of ticks
	
	NM=D64( T, MinN );			// --- Min
	T=T-M64(MinN,NM);			// --- The rest of ticks
	
	NS=D64( T, SecN );			// --- Seconds
	T=T-M64(SecN, NS);

	ms=D64( T, MsN );			// --- mseconds
	T=T-M64(MsN, ms);
	us=D64( T, UsN );			// --- useconds
	T=T-M64(UsN, us);

	
	rtc->Month = NMN;
	rtc->Day = ND;
	rtc->Hour = NH;
	rtc->Min = NM;
	rtc->Sec = NS;
	rtc->msec = ms;
	rtc->usec = us;
	rtc->DOW =  RTC_ComputeDOW(rtc->Day,rtc->Month,rtc->Year);
	
	return T;
}


long long int  Time::RTC_GetKernalTicksOffset(void) 
{
	long long int time=0;	
	volatile long long int  yr,mon,dy,hr,min,sec,frac_sec;
	
	//start of time stamp is january 1 2007
	yr = 7; // 2007	
	mon = 1;	
	dy = 1;	
	hr = 0;	
	min = 0; 	
	sec = 0; 	
	frac_sec = 0; 
	
	// --- Convert  Days:Hours:Min:Sec, FractionSec   into 64 bit timer in [HL LL]

	time += M64( DayN, RTC_YearsToDays(yr) );	
	time += M64( DayN, RTC_MonthsToDays(mon,yr) );		
	time+=M64( DayN, dy );			// --- Day	
	time+=M64( HourN, hr  );		// --- Hours	
	time+=M64( MinN, min );			// --- Min	
	time+=M64( SecN, sec );			// --- Seconds	
	time+=M64( FracN, frac_sec );		// --- 1/100 of a second	
	return time;
}


//months is exclusive
unsigned long long Time::RTC_MonthsToDays(unsigned int month, unsigned int year)
{
	unsigned char DaysInMonth[]={31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31};
	unsigned long long days=0;
	unsigned int i;

	if((year&3)==0) // leap year
		DaysInMonth[1]= 29;

	for(i=0;i!=(month-1);++i)	
		days += DaysInMonth[i];
		
	return days;
}

//months is exclusive							
unsigned long long Time::RTC_YearsToDays(unsigned int year)
{
	unsigned int i;
	unsigned long long days=0;

	for(i=0;i<year;++i)
	{       
		days += 365;
		if((i&3)==0)
			++days;
	}

	return days;
}

long long int Time::RTC_TicksToMonths(long long int t, long long int * months,unsigned long year)
{
	unsigned char DaysInMonth[]={31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31};
	long long int ticks_in_month;
	int i=0;

	if((year&3)==0) // leap year
		DaysInMonth[1]= 29;

	for(;i!=12;++i)
	{
		ticks_in_month = M64( DayN, DaysInMonth[i] );
		if(t >= ticks_in_month )	
			t -= ticks_in_month;
		else
			break;
	}
	*months=i+1;
	return t;
}

int Time::RTC_ComputeDOW(int day, int month,int year)
{
	unsigned char months[] = {0, 0, 3, 3,6,1,4,6,2,5,0,3,5};
	unsigned char dow;

	if((year&3)==0) // leap year
	{
		months[0] = 6;
		months[1] = 2;
	}

	//get bottom 2 decimal digits
	while(year>100)
		year-=100;

	dow = 6 + year + year/4 + months[month] + day;
	return dow % 7;
}

long long int Time::M64 (long long int A, long long int B)
{
	return A*B;
}

long long int Time::D64 (long long int A, long long int B)
{
	return A/B;
}